

Gauss.test <- function(x,y=NULL,mean=0,sd=1,alternative = c("two.sided", "less", "greater")) {
  parameter <-c(mean=mean,sd=sd)
  if(is.null(y)) {
    statistic <- c(T=mean(x))
    isd <- sqrt((sd^2)/length(x))
  } else {
    statistic <- c(T=mean(x)-mean(y))
    sdq <- c(sd^2,sd^2)
    isd <- sqrt(sdq[1]/length(x)+sdq[2]/length(y))
  }
  p1 <- 1-pnorm(statistic,mean=parameter["mean"],sd=isd)
  p2 <- pnorm(statistic,mean=parameter["mean"],sd=isd)
  alternative = match.arg(alternative)
  p.value <- switch(alternative,
                    "two.sided"=2*min(p1,p2),
                    "less"=p2,
                    "greater"=p1
                    )
  structure(list(
                 data.name=deparse(substitute(x)),
                 method="Ein Stichproben Gauss-Test",
                 alternative=alternative,
                 parameter=parameter,
                 statistic=statistic,
                 p.value=p.value
                 ),
            class="htest")
}


gsiStepper <- function(x,update) {
  y = x+update
  if( any(y <= 0) ) {
    lf <- x / -update
    lf <- lf[lf>0]
    lf <- min(lf)/2 
    x+lf*update
  } else y
  
}


fitDirichlet <- function(x,elog=mean(ult(x)),alpha0=rep(1,length(elog)),maxIter=20,n=nrow(x)) {
  alpha <- alpha0
  for( i in 1:maxIter ) {
    E <- digamma(alpha)-digamma(sum(alpha))
    V <- gsi.diagGenerate(trigamma(alpha))-trigamma(sum(alpha))
    delta=sqrt(sum((elog-E)^2))
    if( i==1 )
      delta1=delta
    update <- solve(V,(elog-E))
    alpha <- gsiStepper(alpha,update)
    print(list(alpha=alpha,E=E,V=V,update=update,deltaR=delta/delta1))
  }
  list(alpha=alpha,
       loglikelihood=n*(sum(lgamma(alpha))-lgamma(sum(alpha))+sum(elog*(alpha-1))),
       df=n*(length(elog)-1)-length(elog)
       )
}




acompNormalGOF.test <- function(x,...,method="etest") {
  method <- match.arg(method)
  switch(method,
         "etest"={
           mvnorm.etest(ilr(x),...)
         }
         )
}



gsi.acompUniformityGOF.test<- function(x,
                                samplesize=nrow(x)*20,
                                R=999
                                ) {
  data.name<- paste(deparse(substitute(x)),collapse="",sep="")
  theSample <- runif.acomp(samplesize,ncol(x))
  erg <- acompTwoSampleGOF.test(x,theSample,R=R) 
  erg$data.name<-data.name
  erg$method<-"Compositional test for uniformity"
  erg
}

ccompMultinomialGOF.test<-function(x,
                                   simulate.p.value=TRUE,
                                   R=1999
                                   ){
  chisq.test(unclass(x),simulate.p.value=simulate.p.value,B=R)
}


ccompPoissonGOF.test<-function(x,
                               simulate.p.value=TRUE,
                               R=1999
                               ){
  x <- ccomp(x)
  M <- mean(ccomp(x))
  N <- totals(x)
  erg1 <-chisq.test(unclass(x),simulate.p.value=simulate.p.value,B=R)
  erg2 <-PoissonGOF.test(N,R=R)
  statistic <- min(erg1$p.value , erg2$p.value)
  p.value <- pbeta(statistic,1,2)
  structure(list(
                 data.name=deparse(substitute(x)),
                 method="Count Composition Poission Goodness of fit test",
                 alternative="Count Composition is not a Multi Possion distribution with constant mean",
                 parameter=c(shape1=1,shape2=2),
                 sample=sample,
                 statistic=statistic,
                 p.value=p.value
                 ),
            class="htest")
}

gsi.sortedUniforms <- function(n) {
  n = as.integer(c(n,0)[1])
  .C("gsiKSsortedUniforms",
     n    = as.integer(n),
     data = numeric(n),
     getRng= as.integer(1)
     )$data
}
  
PoissonGOF.test <- function(x,lambda=mean(x),R=999) {
  x <- as.integer(x)  
  Max <- as.integer(max(x))
  ps = dpois(0:Max,lambda)
  statistic <- .C("gsiKSPoisson",
    nd  =as.integer(length(x)),
    data=as.integer(x),
    nps =as.integer(length(ps)),
    ps  =as.numeric(ps),
    n   =integer(length(ps)),
    statistic = numeric(1)
    )$statistic
  sample <- if(R>0)
    .C("gsiKSPoissonSample",
       n=as.integer(length(x)),
       data=numeric(length(x)),
       Nps =as.integer(length(ps)),
       ps  =as.numeric(ps),
       nsamples=as.integer(R),
       statistics=numeric(R)
       )$statistics
  p.value <- sum( statistic<=c(sample) )+1/(R+1)
  structure(list(
                 data.name=deparse(substitute(x)),
                 method="Poisson GOF-Test",
                 alternative="Random variable is not Poisson distributed",
                 parameter=c(),
                 statistic=statistic,
                 p.value=p.value
                 ),
            class="htest")
}
 
acompDirichletGOF.test<- function(x,
                               samplesize=nrow(x)*20,
                               R=999
                               ) {
  df<- fitDirichlet(x)
  n <- length(nrow(x))
  Y <-rDirichlet.acomp(samplesize,df$alpha)
  erg <- acompTwoSampleGOF.test(x,Y,R=R)
  erg$data.name<-deparse(substitute(x))
  erg$method="Compositional test for Dirichlet distribution"
  erg
}


acompTwoSampleGOF.test <- function(x,y,...,method="etest",data=NULL) {
  acompGOF.test(list(x,y),...,method=method)
}

acompGOF.test <- function(x,...) UseMethod("acompGOF.test")

acompGOF.test.formula <- function(formula, data,...,method="etest") {
  
    if (missing(formula) || (length(formula) != 3) || (length(attr(terms(formula[-2]), "term.labels")) != 1)) 
        stop("'formula' missing or incorrect")
    m <- match.call(expand.dots = FALSE)
    if (is.matrix(eval(m$data, parent.frame()))) 
        m$data <- as.data.frame(data)
    m[[1]] <- as.name("model.frame")
    m$... <- NULL
    mf <- eval(m, parent.frame())
    DNAME <- paste(names(mf), collapse = " by ")
    names(mf) <- NULL
    response <- attr(attr(mf, "terms"), "response")
    g <- factor(mf[[-response]])
    DATA <- split(mf[[response]], g)
    names(DATA) <- codes(g)
    y <- do.call("acompGOF.test", c(DATA, list(...,method=method)))
    y$data.name <- DNAME
    y
  }

gsi.AcompGOFEtest <- function(x,distance=FALSE,R=999,...,dname="data") {
  X <- lapply(x,ilr)
  N <- sapply(x,nrow)
  D <- as.data.frame(do.call("rbind",x))
  erg <- eqdist.etest(D,N,R=R)
  erg$data.name <- dname
  erg
}

acompGOF.test.list <- function(x,...,method="etest") {
  method <- match.arg(method)
  data.name <- paste(deparse(substitute(x)),collapse=" ",sep=" ")
  switch(method,
         etest=gsi.AcompGOFEtest(x,...,dname=data.name)
         )
}




gsi.AddMatrices <- function(M) 
  structure(structure(unlist(M),dim=c(length(M[[1]]),length(M))) %*% rep(1,length(M)),dim=dim(M[[1]]))


fitSameMeanDifferentVarianceModel <- function(x) {
  N <- lapply(x,nrow)
  n <- sum(N)
  G <- length(x)
  m <- ncol(x[[1]])
  a1 <- function(SigmaInv,n) n*SigmaInv
  a2 <- function(SigmaInv,n,mean) n*SigmaInv%*%mean
  a3 <- function(mean,Sigma) {D<-mean-M;Sigma+D%o%D} 
  Sigma0 <- lapply(x,var)
  means  <-  lapply(x,function(x) mean(rmult(x)))
  Sigma  <-  Sigma0
  M <- mean(rmult(do.call(cbind,means)))
  
  for(i in 1:20) {
    Mold <- M
    SigmaInv <- lapply(Sigma,solve)
    M <- rmult(solve(gsi.AddMatrices(mapply(a1,SigmaInv,N)),
               gsi.AddMatrices(mapply(a2,SigmaInv,N,means))))
    Sigma <- mapply(a3,means,Sigma0)
    print(norm(M-Mold))
  }
  list(mean=M,vars=Sigma,N=N)
}


acompNormalLocationTest.test <- function(x,g=NULL,var.equal=FALSE) {
  if( !is.null(g) || !is.list(x) )
    Splitted <- split(x,g)
  Splitted <- lapply(Splitted,ilr)
  N <- lapply(Splitted,nrow)
  n <- sum(N)
  G <- length(Splitted)
  m <- ncol(Splitted[[1]])
  css <- function(x) {
    x <- rmult(x)
    x <- unclass(x-mean(x))
    x %*% t(x)
  }
  if( var.equal ) {
    TSS <- css(do.call("rbind",Splitted))
    iRSS <- lapply(Splitted,css)
    tRSS <- gsi.AddMatrices(iRSS)
    logLik <- -0.5*n*(log(det(TSS/n))-log(det(tRSS/n)))
    df <- (G-1)*m
    p.value <- pchisq(logLik,df,lower.tail=FALSE)
    structure(list(
                   data.name=deparse(substitute(x)),
                   method="Compositional Normal Location test with equal variances",
                   alternative="locations of groups are not equal",
                   parameter=c(df=df),
                   statistic=logLik,
                   p.value=p.value
                   ),
              class="htest")
  } else {
    smdvm <- fitSameMeanDifferentVarianceModel(Splitted)$vars
    dmdvm <- lapply(Splitted,function(x) css(x)/nrow(x))
    a0 <- function(n,V1,V2) n*(log(det(V1))-log(det(V2)))
    logLik <- sum(unlist(mapply(a0,N,smdvm,dmdvm)))
    df <- (G-1)*m
    p.value <- pchisq(logLik,df,lower.tail=FALSE)
    structure(list(
                   data.name=deparse(substitute(x)),
                   method="Compositional Normal Location test with different variances",
                   alternative="locations of groups are not equal",
                   parameter=c(df=df),
                   statistic=logLik,
                   p.value=p.value
                   ),
              class="htest")
  }
}

acompNormalSpreadTest.test <- function(x,g=NULL,mean.equal=FALSE) {
  if( !is.null(g) || !is.list(x) )
    Splitted <- split(x,g)
  Splitted <- lapply(Splitted,ilr)
  N <- lapply(Splitted,nrow)
  n <- sum(N)
  G <- length(Splitted)
  m <- ncol(Splitted[[1]])
  css <- function(x) {
    x <- rmult(x)
    x <- unclass(x-mean(x))
    x %*% t(x)
  }
  if( mean.equal ) {
    SS <- css(do.call("rbind",Splitted))/n
    dmdvm <- lapply(Splitted,function(x) css(x)/nrow(x))
    logdetSS<-log(det(SS))
    a0 <- function(n,V2) n*(logdetSS-log(det(V2)))
    logLik <- sum(unlist(mapply(a0,N,dmdvm)))
    dfPerVariance <- m*(m+1)/2
    df <- (G-1)*dfPerVariance
    p.value <- pchisq(logLik,df,lower.tail=FALSE)
    structure(list(
                   data.name=deparse(substitute(x)),
                   method="Compositional Normal Spread test with equal means",
                   alternative="locations of groups are not equal",
                   parameter=c(df=df),
                   statistic=logLik,
                   p.value=p.value
                   ),
              class="htest")
  } else {
    SS <- lapply(Splitted,css)
    MTSS<- gsi.AddMatrices(SS)/n
    logdetMTSS <- log(det(MTSS))
    a0 <- function(n,V2) n*(logdetMTSS-log(det(V2)))
    logLik <- sum(unlist(mapply(a0,n,SS)))
    df <- (G-1)*dfPerVariance
    p.value <- pchisq(logLik,df,lower.tail=FALSE)
    structure(list(
                   data.name=deparse(substitute(x)),
                   method="Compositional Normal Spread test with different means",
                   alternative="locations of groups are not equal",
                   parameter=c(df=df),
                   statistic=logLik,
                   p.value=p.value
                   ),
              class="htest")
  }
}



  
